<?php
	class economic {
		var $json = null;
		var $label_types = ['ok' => '#77d413', 'pending' => '#d4d411', 'failed' => '#d41515'];
		var $cc = ['AT' => 8, 'BE' => 9, 'BG' => 10, 'CY' => 11, 'CZ' => 12, 'DE' => 13, 'EE' => 14, 'ES' => 15, 'FI' => 16, 'FR' => 17, 'GR' => 18, 'HR' => 19, 'HU' => 20, 'IE' => 21, 'IT' => 22, 'LT' => 23, 'LU' => 24, 'LV' => 25, 'MT' => 26, 'NL' => 27, 'PL' => 28, 'PT' => 29, 'RO' => 30, 'SE' => 31, 'SI' => 32, 'SK' => 33];
		var $eu = ['BE', 'EL', 'LT', 'PT', 'BG', 'ES', 'LU', 'RO', 'CZ', 'FR', 'HU', 'SI', 'DK', 'HR', 'MT', 'SK', 'DE', 'IT', 'NL', 'FI', 'EE', 'CY', 'AT', 'SE', 'IE', 'LV', 'PL'];

		public function __construct($json) {
			$this->json = $json;
		}

		public function getCountryCodes() {
			return $this->cc;
		}

		// Idéen er, at man bare bruger ->getVatZone('DK'), og så returneres den rigtige momszone udfra ens hjemland
		public function getVatZone($delivery_country_code, $order = []) {
			// Just in case ..
			if (!$this->json['fields']['home']) {
				$this->json['fields']['home'] = 'DK';
			}

			// no for Norge
			if ($this->json['fields']['use_no_voec']) {
				if ($this->json['fields']['home'] == "DK" && $delivery_country_code == "NO") {
					if ($order['vat_number']) {
						return 3;
					}
					return 1;
				}
			}

			// Man driver forretning i leveringslandet = domestic
			if ($this->json['fields']['home'] == $delivery_country_code) {
				return 1;
			}

			// Slår op om leveringslandet er i listen
			if ($this->cc[$delivery_country_code]) {
				return $this->cc[$delivery_country_code];
			}

			// Slår op om leveringslandet er et EU-land
			if ($this->eu[$delivery_country_code]) {
				return 2;
			}

			// Hvis det ikke er hjemland, i listen, eller i eu-listen, så må det være det store udland
			return 3;
		}

		// e-conomic
		public function get($path) {
			$cache = ROOT."/cache/get.".md5($path.$this->json['fields']['apikey']).".json";
			if (file_exists($cache) && time()-filemtime($cache) < 10) {
				return ['http_code' => 200, 'json' => json_decode(file_get_contents($cache), true)];
			}

			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, "https://restapi.e-conomic.com/".$path."?pagesize=1000");
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HTTPHEADER, [
				"X-AppSecretToken: ".SECRET_TOKEN,
				"X-AgreementGrantToken: ".$this->json['fields']['apikey'],
				"Content-Type: application/json"
			]);

			$json = json_decode(curl_exec($curl), true);
			$info = curl_getinfo($curl);

			curl_close($curl);

			if ($info['http_code'] == 200) {
				file_put_contents($cache, json_encode($json, J));
			}

			file_put_contents(ROOT."/cache/get.".str_replace("/", "", $path).".json", json_encode($json, J));

			return ['http_code' => $info['http_code'], 'json' => $json];
		}

		// e-conomic
		public function post($path, $data) {
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, 'https://restapi.e-conomic.com'.$path);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
			curl_setopt($curl, CURLOPT_HTTPHEADER, [
				"X-AppSecretToken: ".SECRET_TOKEN,
				"X-AgreementGrantToken: ".$this->json['fields']['apikey'],
				"Content-Type: application/json"
			]);

			$json = json_decode(curl_exec($curl), true);
			$info = curl_getinfo($curl);

			curl_close($curl);

			file_put_contents(ROOT."/cache/post.".str_replace("/", "", $path).".txt", json_encode($data, J)."\n\n".json_encode($json, J));

			return ['http_code' => $info['http_code'], 'json' => $json];
		}

		// e-conomic
		public function checkProduct($sku, $name) {
			$return = $this->get("products/".$sku);

			if ($return['http_code'] == 404) {
				$return = $this->createProduct($sku, $name);
			}

			return $return;
		}

		// e-conomic
		public function createProduct($sku, $name) {
			$data = [
		        'productNumber' => (string) $sku,
		        'name' => $name,
		        'productGroup' => ['productGroupNumber' => (int) $this->json['fields']['product_group_id']]
			];

			return $this->post('/products', $data);
		}

		// e-conomic
		public function addContactIfNotExists($customer_number, $contact = []) {
			$contacts = $this->get('/customers/'.$customer_number.'/contacts');

			foreach($contacts['json']['collection'] as $c) {
				if ($c['email'] == $contact['email']) {
					return $c;
				}
			}

			$return = $this->post('/customers/'.$customer_number.'/contacts', $contact);

			return $return['json'];
		}

		// e-conomic
		public function findCustomerNumber($order, &$contact) {
			$customer_number = null;

			switch($this->json['fields']['customer_number']) {
				case "phone":
				$customer_number = (int) substr(preg_replace("/[^0-9]+/", "", $order['phone']), -8);
				break;

				case "customer_id":
				$customer_number = $order['customer_id'];
				break;

				case "fixed":
				$customer_number = $this->json['fields']['fixed_customer_number'];
				break;
			}

			if ($customer_number) {
				$return = $this->get("customers/".$customer_number);
				if ($return['http_code'] == 200) {

					$contact = $this->addContactIfNotExists($return['json']['customerNumber'], [
				        'name'  => $order['order_name'],
				        'email' => $order['email'],
						'phone' => $order['phone']
					]);

					return $return['json'];
				}
			}

			if ($order['vat_number'] || $order['ean_number']) {
				$customer_group_number = $this->json['fields']['customer_group_id_business'];
			}
			else {
				$customer_group_number = $this->json['fields']['customer_group_id_private'];
			}

			$data = [
		        'email'							=> $order['email'],
				'telephoneAndFaxNumber'			=> $order['phone'],
				'zip'							=> $order['order_zipcode'],
				'city'							=> $order['order_city'],
				'address'						=> $order['order_address'],
				'country'						=> $order['order_country_name'],
		        'name'							=> $order['order_company_name'] ? $order['order_company_name'] : $order['order_name'] ,
				'currency'						=> $order['currency'],
				'corporateIdentificationNumber'	=> $order['vat_number'],
				'vatZone'						=> ['vatZoneNumber' => $this->getVatZone($order['del_country_code'], $order)],
				'paymentTerms'					=> ['paymentTermsNumber' => (int) $this->json['fields']['payment_terms_cards']],
		        'customerGroup'					=> ['customerGroupNumber' => (int) $customer_group_number]
			];

			if ($customer_number) {
				$data['customerNumber'] = (int) $customer_number;
			}

			$return = $this->post('/customers', $data);

			$contact = $this->addContactIfNotExists($return['json']['customerNumber'], [
		        'name'  => $order['order_name'],
		        'email' => $order['email'],
				'phone' => $order['phone']
			]);

			return $return['json'];
		}

		// e-conomic
		public function findLayouts() {
			$r = [];

			$return = $this->get("layouts");
			if ($return['http_code'] == 200) {
				foreach($return['json']['collection'] as $layout) {
					$r[$layout['layoutNumber']] = $layout['name'];
				}
			}

			return $r;
		}

		// e-conomic
		public function findPaymentTerms() {
			$r = [];

			$return = $this->get("payment-terms");
			if ($return['http_code'] == 200) {
				foreach($return['json']['collection'] as $term) {
					$r[$term['paymentTermsNumber']] = $term['name'];
				}
			}

			return $r;
		}

		// e-conomic
		public function findCustomerGroups() {
			$r = [];

			$return = $this->get("customer-groups");
			if ($return['http_code'] == 200) {
				foreach($return['json']['collection'] as $term) {
					$r[$term['customerGroupNumber']] = $term['name'];
				}
			}

			return $r;
		}

		// e-conomic
		public function findProductGroups() {
			$r = [];

			$return = $this->get("product-groups");
			if ($return['http_code'] == 200) {
				foreach($return['json']['collection'] as $term) {
					$r[$term['productGroupNumber']] = $term['name'];
				}
			}

			return $r;
		}

		// e-conomic
		public function fire($order_id, $order = null) {
			// Kun hvis den kommer fra add_label
			if (!$order) {
				$order = $this->getOrder($order_id);
			}

			// Looper igennem og siker sig at produkterne findes i e-conomic
			foreach($order['products'] as $product) {
				$this->checkProduct($product['own_id'] ?? $product['product_id'], $product['name']);
			}

			// Sætter data op til e-conomic
			$o = [];

			if ($this->json['fields']['exchange_rate']) {
				$o['exchangeRate'] = $this->json['fields']['exchange_rate'];
			}

			if ($order['ean_number'] || $order['payment_gateway'] == 'other') {
				$payment_terms = (int) $this->json['fields']['payment_terms_other'];
			}
			else {
				$payment_terms = (int) $this->json['fields']['payment_terms_cards'];
			}

			if ($order['ean_number'] || $order['payment_gateway'] == 'other') {
				$layout_number = (int) $this->json['fields']['layout_number_other'];
			}
			else {
				$layout_number = (int) $this->json['fields']['layout_number_cards'];
			}

			$contact = [];
			$customer = $this->findCustomerNumber($order, $contact);

			$o['date']         = date('Y-m-d H:i:s');
			$o['currency']     = $order['currency'];
			$o['netAmount']    = $order['total_price'] - $order['vat'];
			$o['vatAmount']    = $order['vat'];
			$o['grossAmount']  = $order['total_price'];
			$o['layout']       = ['layoutNumber' => $layout_number];
			$o['paymentTerms'] = ['paymentTermsNumber' => $payment_terms];

			$o['notes'] = [
				'heading' => (string) $order['webshop_order_id'],
				'textLine1' => $order['comments']
			];

			$o['customer']   = ['customerNumber'  => $customer['customerNumber']];
			$o['references'] = ['customerContact' => $contact];

			// brok brok brok
			if ($this->json['fields']['use_other_ref']) {
				$o['references']['other'] = (string) $order['webshop_order_id'];
			}

			if ($this->json['fields']['basket_id_in_other_ref']) {
				$o['references']['other'] = (string) $order['basket_id'];
			}

			$o['recipient'] = [
				'name'      => $order['order_company_name'] ? $order['order_company_name'] : $order['order_name'],
				'address'   => $order['order_address'],
				'zip'       => $order['order_zipcode'],
				'city'      => $order['order_city'],
				'country'   => $order['order_country_name'],
				'attention' => $contact,
				'vatZone'   => ['vatZoneNumber' => $this->getVatZone($order['order_country_code'], $order)]
			];
			$o['delivery']  = [
				'name'          => $order['del_company_name'] ? $order['del_company_name'] : $order['del_name'],
				'address'       => $order['del_company_name'] ? $order['del_name']."\n".$order['del_company_name']."\n".$order['del_address'] : $order['del_address'],
				'zip'           => $order['del_zipcode'],
				'city'          => $order['del_city'],
				'country'       => $order['del_country_name'],
				'deliveryTerms' => $order['shipping_method'],
				'vatZone'       => ['vatZoneNumber' => $this->getVatZone($order['del_country_code'], $order)]
			];

			if ($order['ean_number']) {
				$o['recipient']['ean'] = $order['ean_number'];
			};

			if ($order['vat_number']) {
				$o['recipient']['cvr'] = $order['vat_number'];
			}

			$lines = [];
			foreach($order['products'] as $product) {
				$variant = [];
				foreach($product['attributes'] as $attribute) {
					$variant[] = $attribute['val'];
				}

				$lines[] = [
					'product'        => ['productNumber' => $product['own_id'] ?? (string) $product['product_id']],
					'description'    => $product['name'].($variant ? " - ".implode(" ", $variant) : ""),
					'quantity'       => $product['amount'],
					'unitNetPrice'   => (float) number_format($product['unit_price']-$product['unit_vat'], 2, '.', ''),
					'totalNetAmount' => (float) number_format($product['amount'] * ($product['unit_price']-$product['unit_vat']), 2, '.', '')
				];
			}

			// Tilføjer evt. rabat
			if ($total_discount = $order['voucher_discount_amount']) {
				$total_discount = $total_discount * 0.8 * -1; // Bør beregnes anderledes, men vi gemmer ikke moms-rabatten

				$lines[] = [
					'product'        => ['productNumber' => $this->json['fields']['product_id_discount']],
					'description'    => 'Rabat',
					'quantity'       => 1,
					'unitNetPrice'   => (float) number_format($total_discount, 2, '.', ''),
					'totalNetAmount' => (float) number_format($total_discount, 2, '.', '')
				];
			}

			// Tilføjer evt. kampagnerabat
			if ($campaign_discount = $order['campaign_discount']) {
				$campaign_discount = $campaign_discount * 0.8 * -1; // Bør beregnes anderledes, men vi gemmer ikke moms-rabatten

				$lines[] = [
					'product'        => ['productNumber' => $this->json['fields']['product_id_discount']],
					'description'    => 'Kampagnerabat',
					'quantity'       => 1,
					'unitNetPrice'   => (float) number_format($campaign_discount, 2, '.', ''),
					'totalNetAmount' => (float) number_format($campaign_discount, 2, '.', '')
				];
			}

			// Tilføjer levering
			if ($shipping_price = $order['shipping_price']) {
				if ($order['shipping_vat_pct']) {
					$shipping_vat = $shipping_price/(1+($order['shipping_vat_pct']/100))*($order['shipping_vat_pct']/100);
					$shipping_price = $shipping_price - $shipping_vat;
				}

				$lines[] = [
					'product'        => ['productNumber' => $this->json['fields']['product_id_shipping']],
					'description'    => 'Levering: '.$order['shipping_method'],
					'quantity'       => 1,
					'unitNetPrice'   => (float) number_format($shipping_price, 2, '.', ''),
					'totalNetAmount' => (float) number_format($shipping_price, 2, '.', '')
				];
			}

			$o['lines'] = $lines;

			// Overfører som ordre eller faktura
			switch ($this->json['fields']['transfer_as']) {
				case "order":
				$return = $this->post('/orders/drafts', $o);
				break;

				case "invoice":
				$return = $this->post('/invoices/drafts', $o);

				if ($this->json['fields']['auto_book_invoice'] && $return['json']['draftInvoiceNumber']) {
					$return = $this->post('/invoices/booked', ['draftInvoice' => ['draftInvoiceNumber' => $return['json']['draftInvoiceNumber']]]);
				}

				break;
			}

			if ($return['http_code'] == 201) {
				$this->setLabel($order['order_id'], "ok", "Overført til e-conomic ".date('d-m-Y H:i:s'));
			}
			else {
				$this->setLabel($order['order_id'], "failed", "Fejl i overførsel til e-conomic ".date('d-m-Y H:i:s')."\n\n".json_encode($return['json'], J));
			}

			$order_status = [
				'status' => $return['http_code'],
				'time'   => time()
			];

			file_put_contents(ROOT."/data/".$this->json['webshop_id']."/orders/".$order['order_id'].".txt", json_encode($order_status, J));

			return $return;
		}


		// Shoporama
		public function getOrder($order_id) {
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, REST_URL."/order/".$order_id);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

			curl_setopt($curl, CURLOPT_HTTPHEADER, [
				"Authorization: Shoporama ".$this->json['app_token'],
				"User-Agent: e-conomic app",
				"Content-Type: application/json"
			]);

			$return = curl_exec($curl);

			curl_close($curl);

			return json_decode($return, true);
		}

		// Shoporama
		public function setLabel($order_id, $label, $add_note = null) {
			$labels = $this->getLabels();

			$data = [];
			$data['add_label_id'] = $labels[$label];
			unset($labels[$label]);
			$data['remove_label_id'] = array_values($labels);
			$data['add_note'] = $add_note;

			$json = json_encode($data);

			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, REST_URL."/order/".$order_id);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $json);
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PUT");

			curl_setopt($curl, CURLOPT_HTTPHEADER, [
				"Authorization: Shoporama ".$this->json['app_token'],
				"User-Agent: e-conomic app",
				"Content-Type: application/json"
			]);

			$return = curl_exec($curl);

			curl_close($curl);
		}

		// Shoporama - lokal
		public function getLabels() {
			$file = ROOT."/data/".$this->json['webshop_id']."/labels.json";

			if (!file_exists($file)) {
				touch($file, 0777);
				chmod($file, 0777);

				file_put_contents($file, json_encode([], J));
			}

			return json_decode(file_get_contents($file), true);
		}

		// Shoporama - lokal
		public function saveLabels($labels) {
			if ($this->json['webshop_id']) {
				$file = ROOT."/data/".$this->json['webshop_id']."/labels.json";

				if (!file_exists($file)) {
					touch($file, 0777);
					chmod($file, 0777);
				}

				file_put_contents($file, json_encode($labels, J));
			}
		}

		// Shoporama
		public function checkLabels() {
			$local_labels = $this->getLabels();

			// Looper de krævede labels igennem og ser om de er sat lokalt => sætter default til null
			foreach($this->label_types as $label_type=>$color) {
				if (!$local_labels[$label_type]) {
					$local_labels[$label_type] = null;
				}
			}

			// Henter remote labels
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, REST_URL."/order-label");
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HTTPHEADER, [
				"Authorization: Shoporama ".$this->json['app_token'],
				"User-Agent: e-conomic app"
			]);

			$return = curl_exec($curl);

			curl_close($curl);

			$remote_labels = json_decode($return, true);

			// Checker om det lokale ID ikke længere findes = tumpen har fjernet label i admin
			foreach($local_labels as $local_label=>$local_label_id) {
				// Der er en lokal label_id
				if ($local_label_id) {
					$missing = true;
					foreach($remote_labels as $remote_label) {
						if ($local_label_id == $remote_label['order_label_id']) {
							$missing = false;
						}
					}
					if ($missing) {
						$local_labels[$local_label] = null;
					}
				}
			}

			// Søger i remote efter lokale der er null = der skal mappes udfra navn
			foreach($local_labels as $local_label=>$local_label_id) {
				if (!$local_label_id) {
					foreach($remote_labels as $remote_label) {
						if ($remote_label['name'] == 'e-conomic-'.$local_label) {
							$local_labels[$local_label] = $remote_label['order_label_id'];
						}
					}
				}
			}

			$this->saveLabels($local_labels);

			// Opret de labels der ikke findes
			foreach($local_labels as $local_label=>$local_label_id) {
				if (!$local_label_id) {
					$data = [
						"name"  => "e-conomic-".$local_label,
						"color" => $this->label_types[$local_label]
					];

					$curl = curl_init();
					curl_setopt($curl, CURLOPT_URL, REST_URL."/order-label");
					curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
					curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
					curl_setopt($curl, CURLOPT_HTTPHEADER, [
						"Authorization: Shoporama ".$this->json['app_token'],
						"User-Agent: e-conomic app",
						"Content-Type: application/json"
					]);

					$return = curl_exec($curl);
					curl_close($curl);

					$json = json_decode($return, true);
					if ($json['order_label_id']) {
						$local_labels[$local_label] = $json['order_label_id'];
						$this->saveLabels($local_labels);
					}

				}
			}
		}

	}